import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;

import com.schwarcz.enamels.model.*;

import oracle.toplink.essentials.expressions.*;
import oracle.toplink.essentials.internal.expressions.*;
import oracle.toplink.essentials.queryframework.*;

/** */
public class Main {
    
    private EntityManagerFactory emf;

    /* */
    public static void main (String[] args) {

        Main main = new Main();
        main.testReportQuery();
        //main.testJPQL();
    }
    
    public Main () {
        // create EMF
        emf = Persistence.createEntityManagerFactory("j2seEnvironment");
    }
    
    void testReportQuery() {
        oracle.toplink.essentials.ejb.cmp3.EntityManager em = 
            (oracle.toplink.essentials.ejb.cmp3.EntityManager)emf.createEntityManager();

        // Simpilfied JPQL to reproduce the error
        // SELECT ex FROM Exhibition ex, Showpiece AS sh WHERE sh.showId = ex.id

        ExpressionBuilder exBuilder = new ExpressionBuilder(Exhibition.class);
        ReportQuery reportQuery = new ReportQuery(Exhibition.class, exBuilder);
        ExpressionBuilder shBuilder = new ExpressionBuilder(Showpiece.class);
        Expression whereClause = shBuilder.get("showId").equal(exBuilder.get("id"));
        reportQuery.setSelectionCriteria(whereClause);
        reportQuery.addAttribute("ex", exBuilder);
        reportQuery.dontMaintainCache();
        reportQuery.setShouldReturnWithoutReportQueryResult(true);

        List result = (List)em.getActiveSession().executeQuery(reportQuery);
    }
    
    void testJPQL() {
        EntityManager em = emf.createEntityManager();
        
        String jpql =  
            // This JPQL query executes ok
            //"SELECT ex FROM  Showpiece AS sh, Exhibition AS ex " + 
            //"WHERE sh.enamelId = :enamelId AND sh.showId = ex.id ORDER BY ex.year";

            // This JPQL query runs into an exception:
            // Invalid query key [enamelId] in expression.
            "SELECT ex FROM  Exhibition ex, Showpiece AS sh " + 
            "WHERE sh.enamelId = :enamelId AND sh.showId = ex.id ORDER BY ex.year";

        Query query = em.createQuery(jpql);
        query.setParameter("enamelId", "");
        List result = query.getResultList();

        em.close();
    }

}
